"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dynamic_label_common_1 = require("./dynamic-label.common");
var _a = android.graphics, Paint = _a.Paint, Rect = _a.Rect;
var PaintType = (function (_super) {
    __extends(PaintType, _super);
    function PaintType() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return PaintType;
}(Paint));
var RectType = (function (_super) {
    __extends(RectType, _super);
    function RectType() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return RectType;
}(Rect));
var FontMetricsInt = (function (_super) {
    __extends(FontMetricsInt, _super);
    function FontMetricsInt() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return FontMetricsInt;
}(PaintType.FontMetricsInt));
var TextView = android.widget.TextView;
var TextViewType = (function (_super) {
    __extends(TextViewType, _super);
    function TextViewType() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return TextViewType;
}(TextView));
var DynamicLabel = (function (_super) {
    __extends(DynamicLabel, _super);
    function DynamicLabel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DynamicLabel.prototype.getTextExtent = function (text, textSize, maxWidth, maxHeight) {
        var lineSpans = [];
        var wasCut;
        var outRect = new Rect();
        try {
            var tv = this.android;
            var mTextPaint = tv.getPaint();
            mTextPaint.setTextSize(textSize);
            var tr = new TextRect(mTextPaint);
            tr.prepare(text, maxWidth, maxHeight);
            wasCut = tr.wasTextCut();
            lineSpans = tr.textLinesOut();
            for (var i = 0; i < lineSpans.length; i++) {
                var lineText = lineSpans[i].text;
                var y = lineSpans[i].top;
                var textBounds = new Rect();
                mTextPaint.getTextBounds(lineText, 0, lineText.length, textBounds);
                textBounds.offset(0, y);
                outRect.union(textBounds);
            }
        }
        catch (e) {
            console.error(e);
        }
        var width = outRect.width();
        var height = outRect.height();
        return { width: width, height: height, lines: lineSpans, wasCut: wasCut };
    };
    return DynamicLabel;
}(dynamic_label_common_1.Common));
exports.DynamicLabel = DynamicLabel;
var TextRect = (function () {
    function TextRect(paint) {
        this.metrics = null;
        this.Paint = null;
        this.starts = new Array(TextRect.MAX_LINES);
        this.stops = new Array(TextRect.MAX_LINES);
        this.lines = 0;
        this.textHeight = 0;
        this.bounds = new Rect();
        this.text = null;
        this.wasCut = false;
        this.metrics = paint.getFontMetricsInt();
        this.paint = paint;
    }
    TextRect.prototype.prepare = function (text, maxWidth, maxHeight) {
        this.lines = 0;
        this.textHeight = 0;
        this.text = text;
        this.wasCut = false;
        var lineFitInfo = this.findLineFit(text, maxWidth);
        var maximumInLine = lineFitInfo.maxIndex;
        var length = text.length;
        if (length > 0) {
            var lineHeight = -this.metrics.ascent + this.metrics.descent;
            var start = 0;
            var stop_1 = maximumInLine;
            for (;;) {
                for (; start < length; ++start) {
                    var ch = text.charAt(start);
                    if (ch !== '\n' &&
                        ch !== '\r' &&
                        ch !== '\t' &&
                        ch !== ' ')
                        break;
                }
                for (var o = stop_1 + 1; stop_1 < o && stop_1 > start;) {
                    o = stop_1;
                    var lowest = text.indexOf("\n", start);
                    this.paint.getTextBounds(text, start, stop_1, this.bounds);
                    if ((lowest >= start && lowest < stop_1) ||
                        this.bounds.width() > maxWidth) {
                        --stop_1;
                        if (lowest < start ||
                            lowest > stop_1) {
                            var blank = text.lastIndexOf(" ", stop_1);
                            var hyphen = text.lastIndexOf("-", stop_1);
                            if (blank > start &&
                                (hyphen < start || blank > hyphen))
                                lowest = blank;
                            else if (hyphen > start)
                                lowest = hyphen;
                        }
                        if (lowest >= start &&
                            lowest <= stop_1) {
                            var ch = text.charAt(stop_1);
                            if (ch !== '\n' &&
                                ch !== ' ')
                                ++lowest;
                            stop_1 = lowest;
                        }
                        continue;
                    }
                    break;
                }
                if (start >= stop_1)
                    break;
                var minus = 0;
                if (stop_1 < length) {
                    var ch = text.charAt(stop_1 - 1);
                    if (ch === '\n' ||
                        ch === ' ')
                        minus = 1;
                }
                if (this.textHeight + lineHeight > maxHeight) {
                    this.wasCut = true;
                    break;
                }
                this.starts[this.lines] = start;
                this.stops[this.lines] = stop_1 - minus;
                if (++this.lines > TextRect.MAX_LINES) {
                    this.wasCut = true;
                    break;
                }
                if (this.textHeight > 0)
                    this.textHeight += this.metrics.leading;
                this.textHeight += lineHeight;
                if (stop_1 >= length)
                    break;
                start = stop_1;
                stop_1 = length;
            }
        }
        return this.textHeight;
    };
    TextRect.prototype.textLinesOut = function () {
        var top = 0;
        if (this.textHeight === 0)
            return [];
        var outLines = [];
        var before = -this.metrics.ascent;
        var after = this.metrics.descent + this.metrics.leading;
        var y = top;
        var lines = this.lines;
        --lines;
        for (var n = 0; n <= lines; ++n) {
            var t = void 0;
            y += before;
            if (this.wasCut &&
                n === lines &&
                this.stops[n] - this.starts[n] > 3)
                t = this.text.substring(this.starts[n], this.stops[n] - 3).concat("...");
            else
                t = this.text.substring(this.starts[n], this.stops[n]);
            outLines.push({ text: t, top: y });
            y += after;
        }
        return outLines;
    };
    TextRect.prototype.wasTextCut = function () {
        return this.wasCut;
    };
    TextRect.prototype.findLineFit = function (text, cwidth) {
        var index = text.length;
        var width, height;
        while (--index) {
            var bounds = new Rect();
            this.paint.getTextBounds(text, 0, index, bounds);
            if (bounds.width() < cwidth) {
                width = bounds.width();
                height = bounds.height();
                break;
            }
        }
        return { maxIndex: index + 1, width: width, height: height };
    };
    TextRect.MAX_LINES = 256;
    return TextRect;
}());
//# sourceMappingURL=dynamic-label.android.js.map