"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var label_1 = require("tns-core-modules/ui/label");
var platform_1 = require("tns-core-modules/platform");
var scale = platform_1.screen.mainScreen.scale;
var Common = (function (_super) {
    __extends(Common, _super);
    function Common() {
        var _this = _super.call(this) || this;
        _this.on('layoutChanged', function (eventData) {
            var lbl = eventData.object;
            if (_this._isLoaded) {
                setTimeout(function () { lbl.fitText(); });
            }
        });
        _this.on('loaded', function (eventData) {
            var lbl = eventData.object;
            _this._isLoaded = true;
            lbl.lineHeight = 0;
            if (lbl.android) {
                lbl.android.setGravity(17);
                lbl.fitText();
            }
        });
        _this.on('textChange', function (eventData) {
            var lbl = eventData.object;
            if (_this._isLoaded) {
                setTimeout(function () { lbl.fitText(); });
            }
        });
        return _this;
    }
    Common.prototype.findWidth = function () {
        function widthOf(view) {
            var mw = view.getMeasuredWidth() / scale;
            var w = mw || view.width;
            if (typeof w === 'number') {
                return w;
            }
            else {
                if (!view.parent)
                    return 0;
                return widthOf(view.parent);
            }
        }
        return widthOf(this);
    };
    Common.prototype.findHeight = function () {
        function heightOf(view) {
            var mh = view.getMeasuredHeight() / scale;
            var h = mh || view.height;
            if (typeof h === 'number') {
                return h;
            }
            else {
                if (!view.parent)
                    return 0;
                return heightOf(view.parent);
            }
        }
        return heightOf(this);
    };
    Common.prototype.findPadding = function (horz) {
        if (horz === void 0) { horz = false; }
        function propValueOf(view, prop) {
            var v = view.get(prop);
            if (typeof v === 'number') {
                return v;
            }
            else {
                if (!view.parent)
                    return 0;
                return propValueOf(view.parent, prop);
            }
        }
        if (horz) {
            var l = propValueOf(this, 'paddingLeft');
            var r = propValueOf(this, 'paddingRight');
            return l + r;
        }
        var t = propValueOf(this, 'paddingTop');
        var b = propValueOf(this, 'paddingBottom');
        return t + b;
    };
    Common.prototype.findRenderWidth = function () {
        var p = this.findPadding(true);
        return this.findWidth() - p;
    };
    Common.prototype.findRenderHeight = function () {
        var p = this.findPadding(false);
        return this.findHeight() - p;
    };
    Common.prototype.fitText = function () {
        var _this = this;
        var text = this.text;
        var size = 0;
        this.fontSize = 10;
        var minSize = 1;
        var maxWidth = this.findRenderWidth();
        var maxHeight = this.findRenderHeight();
        var testWidth = maxWidth * 0.9;
        var testHeight = maxWidth * 0.6;
        var maxSize = (maxHeight - size) / 2 + size;
        this.fontSize = maxSize;
        size = maxSize;
        var oneLiner = !this.textWrap;
        var wasMultiAt = 0;
        var wasTightMulti = false;
        while (true) {
            var lastSize_1 = size;
            this.fontSize = size;
            var bounds = this.getTextExtent(text, size, maxWidth, maxHeight);
            var badWrap = false;
            var numLines = bounds.lines.length;
            for (var i = 0; i < numLines; i++) {
                var t = bounds.lines[i].text;
                if (i < numLines - 1) {
                    var wc = t.charAt(t.length - 1);
                    badWrap = wc !== ' ' && wc !== ',' && wc !== '-' && wc !== '.' && wc !== '/';
                }
            }
            if (!oneLiner && numLines > 1 && !bounds.wasCut && !badWrap) {
                if (wasTightMulti) {
                    wasMultiAt = (wasMultiAt + size) / 2;
                }
                else
                    wasMultiAt = Math.max(wasMultiAt, size);
                wasTightMulti = badWrap || bounds.height > testHeight * 0.67;
            }
            var nofit = badWrap || bounds.wasCut || (oneLiner && bounds.lines.length > 1) || bounds.width > testWidth || bounds.height > testHeight;
            if (nofit) {
                maxSize = size;
                size = (size - minSize) / 2 + minSize;
            }
            else if (!bounds.wasCut) {
                minSize = size;
                size = (maxSize - size) / 2 + minSize;
            }
            size = Math.round(Math.floor(size * 10) / 10);
            if (size === lastSize_1) {
                break;
            }
        }
        var lastSize = size;
        if (wasMultiAt) {
            size = wasMultiAt;
        }
        size = size * 0.8;
        this.fontSize = 0;
        this.lineHeight = 1;
        this.text = text;
        this.fontSize = size;
        setTimeout(function () {
            var mw = _this.getMeasuredWidth();
            var mh = _this.getMeasuredHeight();
            var bounds = _this.getTextExtent(text, size, mw / scale, mh / scale);
        });
    };
    Common.prototype.getTextExtent = function (text, textSize, maxWidth, maxHeight) {
        console.error('this method should be overridden by platform implementation code' + text);
        var results = { width: maxWidth, height: 0, lines: [], wasCut: false };
        return results;
    };
    Common.prototype.greet = function () {
        return "Hello, Dynamic Label";
    };
    return Common;
}(label_1.Label));
exports.Common = Common;
//# sourceMappingURL=dynamic-label.common.js.map