"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dynamic_label_common_1 = require("./dynamic-label.common");
var platform_1 = require("tns-core-modules/platform");
var scale = platform_1.screen.mainScreen.scale;
var fitRange;
var DBL_MAX = Number.MAX_SAFE_INTEGER;
var FLOAT_MAX = Number.MAX_VALUE;
var CGFLOAT_MAX = FLOAT_MAX;
var DynamicLabel = (function (_super) {
    __extends(DynamicLabel, _super);
    function DynamicLabel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DynamicLabel.prototype.getTextExtent = function (text, textSize, maxWidth, maxHeight) {
        var lineSpans = [];
        var wasCut;
        var outBounds = CGSizeMake(0, 0);
        var originalFontSize = this.fontSize;
        try {
            var uiLabel = this.ios;
            this.fontSize = textSize;
            var tr = new TextRect(uiLabel);
            outBounds = tr.prepare(text, maxWidth, maxHeight);
            wasCut = tr.wasTextCut();
            lineSpans = tr.textLinesOut();
        }
        catch (e) {
            console.error(e);
        }
        this.fontSize = originalFontSize;
        var width = outBounds.width;
        var height = outBounds.height;
        return { width: width, height: height, lines: lineSpans, wasCut: wasCut };
    };
    return DynamicLabel;
}(dynamic_label_common_1.Common));
exports.DynamicLabel = DynamicLabel;
var TextRect = (function () {
    function TextRect(uiLabel) {
        this.starts = new Array(TextRect.MAX_LINES);
        this.stops = new Array(TextRect.MAX_LINES);
        this.lines = 0;
        this.textHeight = 0;
        this.lineHeight = 0;
        this.bounds = new CGSize();
        this.text = null;
        this.wasCut = false;
        this.uiLabel = uiLabel;
    }
    TextRect.prototype.prepare = function (text, maxWidth, maxHeight) {
        this.lines = 0;
        this.textHeight = 0;
        this.text = text;
        this.wasCut = false;
        var lineFitInfo = this.findLineFit(text, maxWidth);
        var maximumInLine = lineFitInfo.maxIndex;
        var length = text.length;
        if (length > 0) {
            this.lineHeight = lineFitInfo.height;
            var start = 0;
            var stop_1 = maximumInLine;
            for (;;) {
                for (; start < length; ++start) {
                    var ch = text.charAt(start);
                    if (ch !== '\n' &&
                        ch !== '\r' &&
                        ch !== '\t' &&
                        ch !== ' ')
                        break;
                }
                for (var o = stop_1 + 1; stop_1 < o && stop_1 > start;) {
                    o = stop_1;
                    var lowest = text.indexOf("\n", start);
                    var subtext = text.substring(start, stop_1);
                    var subfit = this.findLineFit(subtext, maxWidth);
                    if (subfit.width > this.bounds.width)
                        this.bounds.width = subfit.width;
                    this.bounds.height += subfit.height;
                    if ((lowest >= start && lowest < stop_1) ||
                        subfit.width > maxWidth) {
                        --stop_1;
                        if (lowest < start ||
                            lowest > stop_1) {
                            var blank = text.lastIndexOf(" ", stop_1);
                            var hyphen = text.lastIndexOf("-", stop_1);
                            if (blank > start &&
                                (hyphen < start || blank > hyphen))
                                lowest = blank;
                            else if (hyphen > start)
                                lowest = hyphen;
                        }
                        if (lowest >= start &&
                            lowest <= stop_1) {
                            var ch = text.charAt(stop_1);
                            if (ch !== '\n' &&
                                ch !== ' ')
                                ++lowest;
                            stop_1 = lowest;
                        }
                        continue;
                    }
                    break;
                }
                if (start >= stop_1)
                    break;
                var minus = 0;
                if (stop_1 < length) {
                    var ch = text.charAt(stop_1 - 1);
                    if (ch === '\n' ||
                        ch === ' ')
                        minus = 1;
                }
                if (this.textHeight + this.lineHeight > maxHeight) {
                    this.wasCut = true;
                    break;
                }
                this.starts[this.lines] = start;
                this.stops[this.lines] = stop_1 - minus;
                if (++this.lines > TextRect.MAX_LINES) {
                    this.wasCut = true;
                    break;
                }
                this.textHeight += this.lineHeight;
                if (stop_1 >= length)
                    break;
                start = stop_1;
                stop_1 = length;
            }
        }
        return this.bounds;
    };
    TextRect.prototype.textLinesOut = function () {
        var top = 0;
        if (this.textHeight === 0)
            return [];
        var outLines = [];
        var y = top;
        var lines = this.lines;
        --lines;
        for (var n = 0; n <= lines; ++n) {
            var t = void 0;
            t = this.text.substring(this.starts[n], this.stops[n]);
            outLines.push({ text: t, top: y });
            y += this.lineHeight;
        }
        return outLines;
    };
    TextRect.prototype.wasTextCut = function () {
        return this.wasCut;
    };
    TextRect.prototype.findLineFit = function (text, cwidth) {
        var index = text.length;
        var width, height;
        while (--index) {
            var nstext = NSString.alloc().initWithString(text.substring(0, index));
            var fsize = nstext.sizeWithFont(this.uiLabel.font);
            if (fsize.width / scale < cwidth) {
                width = fsize.width;
                height = fsize.height;
                break;
            }
        }
        return { maxIndex: index + 1, width: width, height: height };
    };
    TextRect.MAX_LINES = 256;
    return TextRect;
}());
//# sourceMappingURL=dynamic-label.ios.js.map